import abc
import typing as t


class AsyncHttpClient(t.Protocol):
    @abc.abstractmethod
    async def get(
        self, url: str, *, headers: t.Optional[t.Dict[str, str]] = None
    ) -> dict: ...

    @abc.abstractmethod
    async def post(
        self, url: str, payload: dict, *, headers: t.Optional[t.Dict[str, str]] = None
    ) -> dict: ...

    async def __aenter__(self):
        pass

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        pass


DEFAULT_CLIENT: t.Optional[t.Type[AsyncHttpClient]] = None


try:
    import aiohttp

    class AiohttpAsyncHttpClient(AsyncHttpClient):
        def __init__(self, session: t.Optional[aiohttp.ClientSession] = None) -> None:
            self._session = session

        async def __aenter__(self):
            if self._session is not None:
                raise ValueError("Session already initialized")

            self._session = await aiohttp.ClientSession().__aenter__()

            return self

        async def __aexit__(self, exc_type, exc_val, exc_tb):
            await self._session.close()
            self._session = None

        async def get(
            self, url: str, *, headers: t.Optional[t.Dict[str, str]] = None
        ) -> dict:
            response = await self._session.get(url, headers=headers)
            response.raise_for_status()

            return await response.json()

        async def post(
            self,
            url: str,
            payload: dict,
            *,
            headers: t.Optional[t.Dict[str, str]] = None,
        ) -> dict:
            response = await self._session.post(url, headers=headers, json=payload)
            response.raise_for_status()

            return await response.json()

    DEFAULT_CLIENT = AiohttpAsyncHttpClient
except ImportError:
    pass

try:
    import httpx

    class HttpxAsyncHttpClient(AsyncHttpClient):
        def __init__(self, client: t.Optional[httpx.AsyncClient] = None) -> None:
            self._client = client

        async def __aenter__(self):
            if self._client is not None:
                raise ValueError("Client already initialized")

            self._client = await httpx.AsyncClient().__aenter__()

            return self

        async def __aexit__(self, exc_type, exc_val, exc_tb):
            await self._client.aclose()
            self._client = None

        async def get(
            self, url: str, *, headers: t.Optional[t.Dict[str, str]] = None
        ) -> dict:
            response = await self._client.get(url, headers=headers)

            return response.raise_for_status().json()

        async def post(
            self,
            url: str,
            payload: dict,
            *,
            headers: t.Optional[t.Dict[str, str]] = None,
        ) -> dict:
            response = await self._client.post(url, headers=headers, json=payload)

            return response.raise_for_status().json()

    DEFAULT_CLIENT = HttpxAsyncHttpClient
except ImportError:
    pass


def get_client() -> AsyncHttpClient:
    if DEFAULT_CLIENT is None:
        raise ImportError("Please install aiohttp or httpx")

    return DEFAULT_CLIENT()



import abc
import base64
import logging
import sys
import typing as t

from . import ahttp, domain, http
from .constants import DEFAULT_URL, VERSION

logger = logging.getLogger(__name__)


class BaseClient(abc.ABC):
    def __init__(
        self, login: str, password: str, *, base_url: str = DEFAULT_URL
    ) -> None:
        credentials = base64.b64encode(f"{login}:{password}".encode("utf-8")).decode(
            "utf-8"
        )
        self.headers = {
            "Authorization": f"Basic {credentials}",
            "Content-Type": "application/json",
            "User-Agent": f"android-sms-gateway/{VERSION} (client; python {sys.version_info.major}.{sys.version_info.minor})",
        }
        self.base_url = base_url.rstrip("/")


class APIClient(BaseClient):
    def __init__(
        self,
        login: str,
        password: str,
        *,
        base_url: str = DEFAULT_URL,
        http_client: t.Optional[http.HttpClient] = None,
    ) -> None:
        super().__init__(login, password, base_url=base_url)
        self.http = http_client

    def __enter__(self):
        if self.http is not None:
            raise ValueError("HTTP client already initialized")

        self.http = http.get_client().__enter__()

        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.http.__exit__(exc_type, exc_val, exc_tb)
        self.http = None

    def send(self, message: domain.Message) -> domain.MessageState:
        return domain.MessageState.from_dict(
            self.http.post(
                f"{self.base_url}/message",
                payload=message.asdict(),
                headers=self.headers,
            )
        )

    def get_state(self, _id: str) -> domain.MessageState:
        return domain.MessageState.from_dict(
            self.http.get(f"{self.base_url}/message/{_id}", headers=self.headers)
        )


class AsyncAPIClient(BaseClient):
    def __init__(
        self,
        login: str,
        password: str,
        *,
        base_url: str = DEFAULT_URL,
        http_client: t.Optional[ahttp.AsyncHttpClient] = None,
    ) -> None:
        super().__init__(login, password, base_url=base_url)
        self.http = http_client

    async def __aenter__(self):
        if self.http is not None:
            raise ValueError("HTTP client already initialized")

        self.http = await ahttp.get_client().__aenter__()

        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.http.__aexit__(exc_type, exc_val, exc_tb)
        self.http = None

    async def send(self, message: domain.Message) -> domain.MessageState:
        return domain.MessageState.from_dict(
            await self.http.post(
                f"{self.base_url}/message",
                payload=message.asdict(),
                headers=self.headers,
            )
        )

    async def get_state(self, _id: str) -> domain.MessageState:
        return domain.MessageState.from_dict(
            await self.http.get(f"{self.base_url}/message/{_id}", headers=self.headers)
        )
